searchData={"items":[{"type":"module","title":"OnceMore","doc":"Provides retry functionality with configurable backoff strategies.\n\nOnceMore allows you to easily retry operations that may fail temporarily, such as network \nrequests or distributed system operations. It supports both simple retries and \nstateful retries with an accumulator.\n\nOnceMore is heavily inspired by the [Retry](https://hexdocs.pm/retry) library.\nFor details on the design decisions, see [Problems with Retry](Problems with Retry.md). \nIf you're migrating from Retry, check out the [Migrating from Retry](Migrating from Retry.md).","ref":"OnceMore.html"},{"type":"module","title":"Basic usage - OnceMore","doc":"The most common use case is retrying a function until it succeeds:\n\n    import OnceMore.DelayStreams\n\n    OnceMore.retry(\n      fn -> make_network_call() end,\n      &match?({:error, _}, &1),\n      Stream.take(exponential_backoff(), 5)\n    )","ref":"OnceMore.html#module-basic-usage"},{"type":"module","title":"Features - OnceMore","doc":"* Composable backoff strategies via `OnceMore.DelayStreams`\n  * Flexible retry predicates - specify precisely what conditions to retry\n  * Accumulator support - maintain state between retry attempts","ref":"OnceMore.html#module-features"},{"type":"module","title":"Stateful retries - OnceMore","doc":"For operations that need to maintain state between attempts, use `retry_with_acc/4`:\n\n    OnceMore.retry_with_acc(\n      fn attempts -> {network_call(), attempts + 1} end,\n      fn result, _attempts -> match?({:error, _}, result) end,\n      0,\n      Stream.take(constant_backoff(), 5)\n    )","ref":"OnceMore.html#module-stateful-retries"},{"type":"type","title":"OnceMore.acc/0","doc":"Accumulator used in `retry_with_acc/4` function.","ref":"OnceMore.html#t:acc/0"},{"type":"type","title":"OnceMore.delay/0","doc":"Non-negative integer representing milliseconds to wait between retry attempts.","ref":"OnceMore.html#t:delay/0"},{"type":"type","title":"OnceMore.delays/0","doc":"Sequence of delays between retry attempts.","ref":"OnceMore.html#t:delays/0"},{"type":"type","title":"OnceMore.result/0","doc":"Return value from the function being retried.","ref":"OnceMore.html#t:result/0"},{"type":"function","title":"OnceMore.retry/3","doc":"Retries a function until it succeeds or runs out of retry attempts.\n\nTakes a function to execute, a predicate function that determines if retry is needed, \nand a collection of delay values that determine how much time should pass between attempts.\n\nThe first attempt is made immediately. Delays between subsequent attempts are determined by the provided `delays`.","ref":"OnceMore.html#retry/3"},{"type":"function","title":"Parameters - OnceMore.retry/3","doc":"* `function` - Zero arity function to be called.\n* `should_retry_fn` - Predicate function that receives the result of `function` invocation. Should return a `t:boolean/0` indicating whether retry is needed.\n* `delays` - An enumerable of integers representing milliseconds between retry attempts.\n\nReturns the result of the last `function` invocation after retries are exhausted or success is achieved.","ref":"OnceMore.html#retry/3-parameters"},{"type":"function","title":"Examples - OnceMore.retry/3","doc":"# Retry with exponential backoff, limited to 5 attempts\n    OnceMore.retry(\n      fn -> network_call() end,\n      &match?({:error, _}, &1),\n      Stream.take(exponential_backoff(), 5)\n    )\n\n    # Retry only specific errors with 100ms delays, stopping after 1 second of attempts\n    OnceMore.retry(\n      fn -> network_call() end,\n      &match?({:error, reason} when reason in @reasons, &1)\n      100 |> constant_backoff() |> expiry(1_000)\n    )","ref":"OnceMore.html#retry/3-examples"},{"type":"function","title":"OnceMore.retry_with_acc/4","doc":"Retries a function until it succeeds or runs out of retry attempts passing accumulator between attempts.\n\nTakes a function to execute, a predicate function that determines if retry is needed,\nan initial accumulator value, and a collection of delay values that determine how much time should pass between attempts.\n\nThe first attempt is made immediately. Delays between subsequent attempts are determined by the provided `delays`.","ref":"OnceMore.html#retry_with_acc/4"},{"type":"function","title":"Parameters - OnceMore.retry_with_acc/4","doc":"* `function` - Function that takes an accumulator and returns a tuple of `{result, new_accumulator}`.\n* `should_retry_fn` - Predicate function that receives the result and accumulator. Should return a `t:boolean/0` indicating whether retry is needed.\n* `acc` - Initial accumulator value.\n* `delays` - An enumerable of integers representing milliseconds between retry attempts.\n\nReturns a tuple containing the result of the last `function` invocation and final accumulator value after retries are exhausted or success is achieved.","ref":"OnceMore.html#retry_with_acc/4-parameters"},{"type":"function","title":"Examples - OnceMore.retry_with_acc/4","doc":"# Retry with exponential backoff, tracking attempt count\n    OnceMore.retry_with_acc(\n      fn count -> {network_call(), count + 1} end,\n      fn result, _count -> match?({:error, _}, result) end,\n      0,\n      Stream.take(exponential_backoff(), 5)\n    )\n\n    # Retry with constant backoff, accumulating errors\n    OnceMore.retry_with_acc(\n      fn errors -> \n        case network_call() do\n          {:error, reason} = err -> {err, [reason | errors]}\n          success -> {success, errors}\n        end\n      end,\n      fn result, _errors -> match?({:error, _}, result),\n      [],\n      Stream.take(constant_backoff(), 5)\n    )","ref":"OnceMore.html#retry_with_acc/4-examples"},{"type":"module","title":"OnceMore.DelayStreams","doc":"Functions to produce or transform streams of delay values.\n\nYou can import the module for convenient access:\n\n    import OnceMore.DelayStreams\n\nCommon delay patterns can be composed using stream transformations:\n\n    # Exponential growth with 50ms initial delay, randomized, capped at 1 second, stop after 5 attempts\n    50\n    |> exponential_backoff()\n    |> randomize(0.2)\n    |> cap(1_000)\n    |> Stream.take(5)\n\n    # Linear growth with 100ms initial delay, with jitter, stop after 30 seconds\n    100\n    |> linear_backoff(50)\n    |> jitter()\n    |> expiry(30_000)\n\n> #### Infinite retries {: .warning}\n>\n> Most functions in this module produce infinite sequences.\n>\n> Always bound them with either `Stream.take/2` or `expiry/3` to prevent infinite retries:\n>\n>     # Limit by number of attempts\n>     exponential_backoff() |> Stream.take(5)\n>\n>     # Limit by total time budget\n>     constant_backoff(100) |> expiry(30_000)","ref":"OnceMore.DelayStreams.html"},{"type":"module","title":"Delay streams are not special! - OnceMore.DelayStreams","doc":"Any `Enumerable` of non-negative integers can be used as `delays`:\n\n     # Fixed delays with a list\n     OnceMore.retry(fn -> ... end, &should_retry?/1, [100, 200, 300])\n\n     # Increasing delays with a range\n     OnceMore.retry(fn -> ... end, &should_retry?/1, 100..500//100)\n\nThe design and implementation of these delay streams are derived\nfrom [ElixirRetry](https://github.com/safwank/ElixirRetry) project\nmade by Safwan Kamarrudin and other [contributors](https://github.com/safwank/ElixirRetry/graphs/contributors).","ref":"OnceMore.DelayStreams.html#module-delay-streams-are-not-special"},{"type":"function","title":"OnceMore.DelayStreams.cap/2","doc":"Returns a stream that is the same as `delays` except that the delays never exceed `max`.\n\nThis allow capping the delay between attempts to some max value.","ref":"OnceMore.DelayStreams.html#cap/2"},{"type":"function","title":"Examples - OnceMore.DelayStreams.cap/2","doc":"# Uncapped \n    iex> 100 |> linear_backoff(100) |> Enum.take(5)\n    [100, 200, 300, 400, 500]\n    # Capped \n    iex> 100 |> linear_backoff(100) |> cap(250) |> Enum.take(5)\n    [100, 200, 250, 250, 250]","ref":"OnceMore.DelayStreams.html#cap/2-examples"},{"type":"function","title":"OnceMore.DelayStreams.constant_backoff/1","doc":"Returns a constant stream of delays.","ref":"OnceMore.DelayStreams.html#constant_backoff/1"},{"type":"function","title":"Examples - OnceMore.DelayStreams.constant_backoff/1","doc":"iex> Enum.take(constant_backoff(), 5)\n    [100, 100, 100, 100, 100]\n\n    iex> 250 |> constant_backoff() |> Enum.take(5)\n    [250, 250, 250, 250, 250]","ref":"OnceMore.DelayStreams.html#constant_backoff/1-examples"},{"type":"function","title":"OnceMore.DelayStreams.expiry/3","doc":"Returns a delay stream that is the same as `delays` except it limits the total life span of the stream to `time_budget`.\n\nThis calculation takes the execution time of the block being retried into account.\n\nThe execution of the code within the block will not be interrupted, so\nthe total time of execution may run over the `time_budget` depending on how\nlong a single try will take.\n\nOptionally, you can specify a minimum delay so the smallest value doesn't go\nbelow the threshold.","ref":"OnceMore.DelayStreams.html#expiry/3"},{"type":"function","title":"Examples - OnceMore.DelayStreams.expiry/3","doc":"100\n    |> constant_backoff()\n    |> expiry(500)\n    |> Stream.each(&Process.sleep/1)\n    |> Enum.sum()\n    500","ref":"OnceMore.DelayStreams.html#expiry/3-examples"},{"type":"function","title":"OnceMore.DelayStreams.exponential_backoff/2","doc":"Returns a stream of delays that increase exponentially.\n\nResulting values are rounded with `Kernel.round/1` in case floating point factor is used.","ref":"OnceMore.DelayStreams.html#exponential_backoff/2"},{"type":"function","title":"Examples - OnceMore.DelayStreams.exponential_backoff/2","doc":"iex> exponential_backoff() |> Enum.take(5)\n    [10, 20, 40, 80, 160]\n\n    iex> 100 |> exponential_backoff(1.5) |> Enum.take(5)\n    [100, 150, 225, 338, 507]","ref":"OnceMore.DelayStreams.html#exponential_backoff/2-examples"},{"type":"type","title":"OnceMore.DelayStreams.factor/0","doc":"","ref":"OnceMore.DelayStreams.html#t:factor/0"},{"type":"function","title":"OnceMore.DelayStreams.jitter/1","doc":"Returns a stream in which each element of `delays` is randomly adjusted to a number\nbetween 1 and the original delay.","ref":"OnceMore.DelayStreams.html#jitter/1"},{"type":"function","title":"Examples - OnceMore.DelayStreams.jitter/1","doc":"# Without jitter\n    iex> 10 |> linear_backoff(10) |> Enum.take(5)\n    [10, 20, 30, 40, 50]\n\n    # With jitter\n    10 |> linear_backoff(10) |> jitter() |> Enum.take(5)\n    [8, 14, 28, 27, 15]","ref":"OnceMore.DelayStreams.html#jitter/1-examples"},{"type":"function","title":"OnceMore.DelayStreams.linear_backoff/2","doc":"Returns a stream of delays that increase linearly.\n\nResulting values are rounded with `Kernel.round/1` in case floating point factor is used.","ref":"OnceMore.DelayStreams.html#linear_backoff/2"},{"type":"function","title":"Examples - OnceMore.DelayStreams.linear_backoff/2","doc":"iex> 10 |> linear_backoff(10) |> Enum.take(5)\n    [10, 20, 30, 40, 50]\n\n    iex> 100 |> linear_backoff(50) |> Enum.take(5)\n    [100, 150, 200, 250, 300]","ref":"OnceMore.DelayStreams.html#linear_backoff/2-examples"},{"type":"function","title":"OnceMore.DelayStreams.randomize/2","doc":"Returns a stream in which each element of `delays` is randomly adjusted no more than `proportion` of the delay.","ref":"OnceMore.DelayStreams.html#randomize/2"},{"type":"function","title":"Examples - OnceMore.DelayStreams.randomize/2","doc":"100 |> linear_backoff(50) |> randomize() |> Enum.take(5)\n    [102, 141, 203, 226, 272]\n\n    100 |> linear_backoff(50) |> randomize(0.5) |> Enum.take(5)\n    [130, 135, 106, 317, 191]","ref":"OnceMore.DelayStreams.html#randomize/2-examples"},{"type":"extras","title":"Migrating from Retry","doc":"# Migrating from Retry\n\n> #### Note {: .info}\n> \n> The following examples demonstrate how to preserve exact behavior when migrating from Retry to OnceMore.\n> However, you may want to consider restructuring your retry logic instead of maintaining direct equivalence.","ref":"migrating-from-retry.html"},{"type":"extras","title":"retry/2 - Migrating from Retry","doc":"```elixir\n# Retry\nuse Retry\nretry with: delays do\n  call_service()\nend\n\n# OnceMore\nimport OnceMore.DelayStreams\nOnceMore.retry(\n  &call_service/0,\n  &(&1 == :error or match?({:error, _reason}, &1)),\n  delays\n)\n```\n\n> #### RuntimeError {: .warning}\n> \n> This and the following examples ignore the fact that, by default, __Retry__ rescues `RuntimeError` exception and treats it as a reason to retry. See [Problems with Retry](problems-with-retry.html#rescue-runtimeerror-as-default) for why this behavior is undesirable.","ref":"migrating-from-retry.html#retry-2"},{"type":"extras","title":"Atoms and error tuples - Migrating from Retry","doc":"> If the block returns any of the atoms specified in `atoms`, a retry will be attempted. Other atoms or atom-result tuples will not be retried. If `atoms` is not specified, it defaults to `[:error]`.\n\n_from [Retry](https://hexdocs.pm/retry/Retry.html#retry/2) docs_\n\nIf you relied on `atoms` option you can adjust `should_retry_fn` as necessary:\n\n```elixir\n# Retry\nretry with: delays, atoms: [:error, :custom_error] do\n  call_service()\nend\n\n# OnceMore\n@errors [:error, :custom_error]\nOnceMore.retry(\n  &call_service/0,\n  &(&1 in @errors or match?({error, _reason} when error in @errors, &1)),\n  delays\n)\n```","ref":"migrating-from-retry.html#atoms-and-error-tuples"},{"type":"extras","title":"Exceptions - Migrating from Retry","doc":"> Similary, if the block raises any of the exceptions specified in `rescue_only`, a retry will be attempted. Other exceptions will not be retried. If `rescue_only` is not specified, it defaults to `[RuntimeError]`.\n\n_from [Retry](https://hexdocs.pm/retry/Retry.html#retry/2) docs_\n\nIf you relied on __Retry__ catching exceptions, with __OnceMore__ you should catch your exceptions and translate them to values yourself:\n\n\n```elixir\n# Imagine call_service/0 can raise `CustomException`\n\n# Retry\nretry with: delays, rescue_only: [CustomException] do\n  call_service()\nend\n\n# OnceMore\nfn ->\n  try do\n    call_service()\n  rescue\n    e in CustomException -> {:error, e}\n  end\nend\n|> OnceMore.retry(&match?({:error, _reason}, &1), delays)\n# Retry raises unresolved exceptions after delays are exhausted\n|> then(fn \n  {:error, e} when is_exception(e, CustomException) -> raise e\n  result -> result\nend)\n```","ref":"migrating-from-retry.html#exceptions"},{"type":"extras","title":"after and else - Migrating from Retry","doc":">   The `after` block evaluates only when the `do` block returns a valid value before timeout.\n> On the other hand, the `else` block evaluates only when the `do` block remains erroneous after timeout.\n\n_from [Retry](https://hexdocs.pm/retry/Retry.html#retry/2) docs_\n\nIf you relied on __Retry__ remapping your results with `after` and `else` blocks, with __OnceMore__ you should remap them yourself:\n\n```elixir\nretry with: delays do\n  call_service()\nafter\n  {:ok, _value} -> :ok\nelse\n  {:error, _reason} -> :error\nend\n\n# OnceMore\n&call_service/0\n|> OnceMore.retry(&match?({:error, _reason}, &1), delays)\n|> then(fn\n    {:ok, _value} -> :ok \n    {:error, _reason} -> :error\nend)\n```","ref":"migrating-from-retry.html#after-and-else"},{"type":"extras","title":"retry_while/2 - Migrating from Retry","doc":"OnceMore returns both last result and accumulator while Retry returns only the accumulator.\nYou need to adjust return values if you want to preserve that behavior.\n\n```elixir\n# Retry\nretry_while acc: 0, with: delays do\n  acc ->\n    case call_service() do\n      %{\"errors\" => true} -> {:cont, acc + 1}\n      result -> {:halt, result}\n    end\nend\n\n# OnceMore\nfn acc ->\n  case call_service() do\n    %{\"errors\" => true} -> {:error, acc + 1}\n    result -> {:ok, result}\n  end\nend\n|> OnceMore.retry_with_acc(\n  fn result, _acc -> result == :error end,\n  0,\n  delays\n)\n|> then(fn\n  {:ok, result} -> result\n  {:error, acc} -> acc\nend)\n```","ref":"migrating-from-retry.html#retry_while-2"},{"type":"extras","title":"wait/2 - Migrating from Retry","doc":"__Retry__ decides if retry is needed based on result being \"falsey\" (`x in [false, nil]`). You can achieve the same behavior by passing `Kernel.!/1` as a predicate to __OnceMore__.\n\nKeep in mind that __Retry__ wraps result in `:ok/:error` tuple depending on if it's \"truthy\" or \"falsey\". You need to wrap it yourself if you want to preserve that behavior.\n\n```elixir\n# Retry\nwait delays do\n  get_available_service()\nend\n\n# OnceMore\n&get_available_service/0\n|> OnceMore.retry(&Kernel.!/1, delays)\n|> then(fn result ->\n  if result do\n    {:ok, result}\n  else\n    {:error, result}\n  end\nend)\n```","ref":"migrating-from-retry.html#wait-2"},{"type":"extras","title":"Problems with Retry","doc":"# Problems with Retry\n\nThis document highlights suboptimal decisions made in the __Retry__ library. The analysis is based on version __0.18.0__, which is the latest version as of early 2025. Future versions of the library may address some or all of these issues. I will update this document accordingly when they do.\n\n> #### Info {: .info}\n>\n> I would like to point out that the __Retry__ library was created almost a decade ago (version 0.1.0 released on May 4, 2016), and there was no clear consensus then on many things that seem obvious today.","ref":"problems-with-retry.html"},{"type":"extras","title":"Retry conditions - Problems with Retry","doc":"Retry logic is typically implemented when making network calls - to databases, Redis, or external HTTP services. Possible responses from such calls can be categorized into three types:\n\n1. Service received the request, understood it, and executed it\n2. Service received the request but either didn't understand it or refused to execute it\n3. Service never received the request\n\nOnly the first response can be considered successful, which is usually represented in Elixir as `:ok` or `{:ok, result}`. The other two categories are represented as `{:error, reason}`, where programmers must to examine the `reason` field to determine the specific category.\n\nHere's a real example from the `command/3` function in [Redix](https://hexdocs.pm/redix/Redix.html#command/3) library:\n\n> The return value is `{:ok, response}` if the request is successful and the\n  response is not a Redis error. `{:error, reason}` is returned in case there's\n  an error in the request (such as losing the connection to Redis in between the\n  request). `reason` can also be a `Redix.Error` exception in case Redis is\n  reachable but returns an error (such as a type error).\n\nHere, `{:ok, response}` is a successful result, `{:error, %Redix.Error{}}` means that Redis received the request but refused to execute it, and all other `{:error, reason}` cases indicate that Redis did not receive the request.\n\nIn the context of retries, it's important to note that it only makes sense to retry requests in the third category.\n\n__Retry__ doesn't provide an easy way to express this. This code will trigger retries even if the initial command is formed incorrectly:\n\n```elixir\nretry with: delays do\n  Redix.command(conn, command)\nend\n```\n\nA working approach for __Retry__ requires converting the relevant error category in the `do` block to a separate one and configuring the library to retry only that category:\n\n```elixir\nretry with: delays, atoms: [:retryable] do\n  case Redix.command(conn, command) do\n    {:error, reason} when not is_struct(reason, Redix.Error) ->\n      {:retryable, reason}\n\n    result ->\n      result\n  end\n  else\n    {:retryable, reason} -> {:error, reason}\n  end\n```\n\nWith __OnceMore__, programmers can precisely configure which errors should trigger retries:\n\n```elixir\nOnceMore.retry(\n  fn -> Redix.command(conn, command) end,\n  &match?({:error, reason} when not is_struct(reason, Redix.Error), &1),\n  delays\n)\n```","ref":"problems-with-retry.html#retry-conditions"},{"type":"extras","title":"Exception handling - Problems with Retry","doc":"In addition to atoms and tuples, __Retry__ can trigger retries based on exception types raised by the specified block of code.\n\n> ... if the block raises any of the exceptions specified in `rescue_only`, a retry will be attempted. Other exceptions will not be retried. If `rescue_only` is not specified, it defaults to `[RuntimeError]`.\n\n_from [Retry](https://hexdocs.pm/retry/Retry.html#retry/2) docs_\n\nUsing exceptions to control program flow can be considered [an anti-pattern](`e:elixir:design-anti-patterns.html#exceptions-for-control-flow`) (although the anti-pattern wording doesn't definitively classify __Retry__ as such).\n\n__Retry__'s exception handling implementation has several specific issues.","ref":"problems-with-retry.html#exception-handling"},{"type":"extras","title":"Rescue RuntimeError as default - Problems with Retry","doc":"Using `RuntimeError` as the default is problematic because this exception is typically used as a runtime assertion. For example, code could validate its configuration and raise this exception if it detects an error:\n\n```elixir\ndef call_service(opts) do\n    if opts[:option_1] && opts[:option_2] do\n        raise \"Passing :option_1 and :option_2 together is invalid\"\n    end\nend\n```\n\nRetrying this code with the same configuration is pointless because the result won't change.\n\nTo avoid this behavior, programmers need to pass an empty list to the `rescue_only` option.","ref":"problems-with-retry.html#rescue-runtimeerror-as-default"},{"type":"extras","title":"Loss of stacktrace - Problems with Retry","doc":"__Retry__ \"loses\" the stacktrace for exceptions listed in `rescue_only`.\n\nFor example, this code:\n```elixir\nMix.install([:retry])\n\ndefmodule M do\n  use Retry\n\n  def run do\n    retry with: [100] do\n      call_service()\n    end\n  end\n\n  defp call_service do\n    if :erlang.phash2(1, 1) == 0 do\n      raise \"oops!\"\n    end\n  end \nend\n\nM.run()\n```\nwill return an incomplete stacktrace\n```sh\n$ elixir script.exs        \n** (RuntimeError) oops!\n    script.exs:7: M.run/0\n    script.exs:19: (file)\n```\n\nUsing `rescue_only: []` allows getting the full stacktrace:\n```sh\n$ elixir script.exs\n** (RuntimeError) oops!\n    script.exs:14: M.call_service/0\n    script.exs:8: anonymous fn/0 in M.run/0\n    (elixir 1.18.1) lib/enum.ex:4964: Enumerable.List.reduce/3\n    (elixir 1.18.1) lib/stream.ex:1041: Stream.do_transform_inner_list/7\n    (elixir 1.18.1) lib/enum.ex:2600: Enum.reduce_while/3\n    script.exs:7: M.run/0\n    script.exs:19: (file)\n```","ref":"problems-with-retry.html#loss-of-stacktrace"},{"type":"extras","title":"Macro usage - Problems with Retry","doc":"The __Retry__ library uses macros for its operation, generating unexpectedly large amounts of code at macro call sites.\n\nFor example, you might write:\n```elixir\nretry with: Stream.take(constant_backoff(), 10) do\n  Enum.random([:ok, :error])\nend\n```\n\nAnd after expanding the `retry/2` macro, you get:\n```elixir\nfun = fn ->\n  try do\n    case Enum.random([:ok, :error]) do\n      {atom, _} = result ->\n        if atom in [:error] do\n          {:cont, result}\n        else\n          {:halt, result}\n        end\n\n      result ->\n        if is_atom(result) and result in [:error] do\n          {:cont, result}\n        else\n          {:halt, result}\n        end\n    end\n  rescue\n    e ->\n      if e.__struct__ in [RuntimeError] do\n        {:cont, {:exception, e}}\n      else\n        reraise e, __STACKTRACE__\n      end\n  end\nend\n\n(\n  delays = Stream.take(constant_backoff(), 10)\n  [0] |> Stream.concat(delays)\n)\n|> Enum.reduce_while(nil, fn delay, _last_result ->\n  :timer.sleep(delay)\n  fun.()\nend)\n|> case do\n  {:exception, e} ->\n    case e do\n      e when is_exception(e) -> raise e\n      e -> e\n    end\n\n  e = {atom, _} when atom in [:error] ->\n    case e do\n      e when is_exception(e) -> raise e\n      e -> e\n    end\n\n  e when is_atom(e) and e in [:error] ->\n    case e do\n      e when is_exception(e) -> raise e\n      e -> e\n    end\n\n  result ->\n    case result do\n      result -> result\n    end\nend\n```\n\nWhile 50 additional lines of code is not a significant concern, it should be noted that this amount of code will be generated with each new `retry/2` call. This can have a negative impact on project compilation time. Furthermore, each expanded version includes all possible error handling variants: `:error` atoms, `{:error, reason}` tuples, and exceptions. However, in practice each call likely works with only one error type.\n\nAlso, the `retry/2` macro does nothing special that can't be done by a function call (as demonstrated by __OnceMore__), which means it is [an anti-pattern](`e:elixir:macro-anti-patterns.html#unnecessary-macros`).","ref":"problems-with-retry.html#macro-usage"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}